/* ------------------------------------------------------------------------- 
 * $Id: gomp.c,v 1.1 2004/04/08 21:07:23 jwp Exp $
 * ------------------------------------------------------------------------- */

/*
 * Copyright (c) 2004
 *				see AUTHORS list
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* ------------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "libgomp.h"
#include "gomp.h"

#ifdef __cplusplus
extern	"C"	{ 
#endif

/* structure containing the current state.  */
struct t_gomp_state gomp_state; 

/* ------------------------------------------------------------------------- */

void gomp_strip( char *s )
{
	if (s == NULL)
		return;
	char tmp[ENV_BUFFER_SIZE]; 
	int i, start, stop;

	strncpy( tmp, s, ENV_BUFFER_SIZE-1 );
	tmp [ENV_BUFFER_SIZE-1] = '\0';

	for ( start=0; start<strlen( tmp ); start++ )
		if ( tmp[start] != ' ' && tmp[start] != '\t' )
			break;

	for ( stop=(strlen(tmp)-1); stop>start; stop-- )
		if ( tmp[stop] != ' ' && tmp[stop] != '\t' )
			break;

	for ( i=start; i<=stop; i++ )
		s[i-start] = tmp[i];

	s[i-start] = '\0';
}

/* ------------------------------------------------------------------------- */

int gomp_init()
{
	gomp_state.schedule = GOMP_DEFAULT_SCHEDULE;
	gomp_state.schedule_chunksize = GOMP_DEFAULT_SCHEDULE_CHUNKSIZE;
	gomp_state.default_number_of_threads = GOMP_DEFAULT_NUMBER_THREADS;
	gomp_state.dynamic_adjustment = GOMP_DEFAULT_DYNAMIC_ADJUSTMENT;
	gomp_state.nested_parallism = GOMP_DEFAULT_NESTED_PARALLISM;

	int error = 0;
	char *schedule = NULL;
    if ( ( schedule = getenv( "OMP_SCHEDULE" ) ) != NULL )
	{
    	char tmp[ENV_BUFFER_SIZE];
	    char *pchunk_size = NULL;
		char *schedule_kind = NULL;
		int chunk_size = 0;
	    strncpy( tmp, schedule, ENV_BUFFER_SIZE-1 );
		tmp [ENV_BUFFER_SIZE-1] = '\0';
		schedule_kind = strtok( tmp, "," );
		if ( (pchunk_size = strtok( NULL, "," )) != NULL )
			gomp_strip(pchunk_size);
		gomp_strip(schedule_kind);
		if ( pchunk_size ) 
			chunk_size = atoi( pchunk_size );

		if (!strcmp (schedule_kind, "static"))
  			gomp_state.schedule = GOMP_STATIC;

		else if (!strcmp (schedule_kind, "dynamic"))
  			gomp_state.schedule = GOMP_DYNAMIC;

		else if (!strcmp (schedule_kind, "guided"))
  			gomp_state.schedule = GOMP_GUIDED;

		else if (!strcmp (schedule_kind, "runtime"))
  			gomp_state.schedule = GOMP_RUNTIME;

		else
		{
    		error = GOMP_ERR_ENV_SCHEDULE;
			if ( gomp_is_debug() )
				fprintf( stderr, "gomp - %s\n", gomp_get_errstr( error ) );
		}
		if (chunk_size > 0)
  			gomp_state.schedule_chunksize = (unsigned int) chunk_size;

	}

	char *pnum_threads = NULL;
    if ( ( pnum_threads = getenv( "OMP_NUM_THREADS" ) ) != NULL )
	{
		int num_threads = 0;
		num_threads = atoi( pnum_threads );
		if ( num_threads > 0 )
			gomp_state.default_number_of_threads = (unsigned int) num_threads;
	}

	char *omp_dynamic = NULL;
    if ( ( omp_dynamic = getenv( "OMP_DYNAMIC" ) ) != NULL )
	{
		gomp_strip(omp_dynamic);
		if (!strcmp (omp_dynamic, "TRUE"))
  			gomp_state.dynamic_adjustment = TRUE;

		else if (!strcmp (omp_dynamic, "FALSE"))
  			gomp_state.dynamic_adjustment = FALSE;

		else
		{
    		error = GOMP_ERR_ENV_DYNAMIC;
			if ( gomp_is_debug() )
				fprintf( stderr, "gomp - %s\n", gomp_get_errstr( error ) );
		}
	}

	char *omp_nested = NULL;
    if ( ( omp_nested = getenv( "OMP_NESTED" ) ) != NULL )
	{
		gomp_strip(omp_nested);
		if (!strcmp (omp_nested, "TRUE"))
  			gomp_state.nested_parallism = TRUE;

		else if (!strcmp (omp_nested, "FALSE"))
  			gomp_state.nested_parallism = FALSE;

		else
		{
    		error = GOMP_ERR_ENV_NESTED;
			if ( gomp_is_debug() )
				fprintf( stderr, "gomp - %s\n", gomp_get_errstr( error ) );
		}
	}

	return error;

}

/* ------------------------------------------------------------------------- */

void gomp_print_state ( FILE *out, char *indent )
{
	/* 
	 * Prints each element of the GOMP state structure to the specified
	 * stream. Each element will be indented by the 'indent' string.
	 */

	char *ind;
	ind = indent ? indent : "";
	char *tmp = NULL;
	switch (gomp_state.schedule)
	{
		case GOMP_DYNAMIC:
			tmp = "DYNAMIC";
			break;
		case GOMP_STATIC:
			tmp = "STATIC";
			break;
		case GOMP_RUNTIME:
			tmp = "RUNTIME";
			break;
		case GOMP_GUIDED:
			tmp = "GUIDED";
			break;
	}
	fprintf( out, "%sschedule: %s\n", ind, tmp );
	fprintf( out, "%sschedule_chunksize: %d\n", ind,
			gomp_state.schedule_chunksize ); 
	fprintf( out, "%sdefault_number_of_threads: %d\n", ind,
			gomp_state.default_number_of_threads );
	fprintf( out, "%sdynamic_adjustment: %d\n", ind,
			gomp_state.dynamic_adjustment );
	fprintf( out, "%snested_parallism: %d\n", ind,
			gomp_state.nested_parallism ); 
}

/* ------------------------------------------------------------------------- */

int gomp_status = 0;

static int gomp_debug = 0;

/* Error messages, keep in sync with error codes in header file !!! */
static char *gomp_err_strings[] = {
	"unknown schedule-kind",
	"unknown dynamic",
	"unknown nested",
	"num_thread must be positive",
	NULL
};

void gomp_set_debug( int flag ) 
{
	gomp_debug = flag;
}

int gomp_is_debug( void ) 
{
	return gomp_debug;
}

char *gomp_get_errstr( int err ) 
{
	if ( err == 0 )
		return "success";
	else if ( err <= GOMP_ERR_MIN || err >= GOMP_ERR_MAX )
		return "unkown error";
	else
		return gomp_err_strings[err - GOMP_ERR_MIN - 1];  
}

/* ------------------------------------------------------------------------- */

#ifdef __cplusplus
}
#endif

/* ------------------------------------------------------------------------- */

