/* ------------------------------------------------------------------------- 
 * $Id: gomp.h,v 1.1 2004/04/08 21:07:23 jwp Exp $
 * ------------------------------------------------------------------------- */

/*
 * Copyright (c) 2004
 *				see AUTHORS list
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* ------------------------------------------------------------------------- */

#ifndef _GOMP_H
#define _GOMP_H

#ifdef __cplusplus
extern "C" {
#endif

#ifndef HAVE_BOOLEAN
typedef int boolean;
#endif
#ifndef FALSE
#define FALSE	0
#endif
#ifndef TRUE
#define TRUE    1
#endif

#define	ENV_BUFFER_SIZE 25 /* upper bound on the length of the environment 
							  variable buffer */

/* ------------------------------------------------------------------------- */

/* Error codes, corr. error strings are defined in the corr. c-file */
enum GOMP_ERR {
	GOMP_ERR_MIN = 100,

	GOMP_ERR_ENV_SCHEDULE,
	GOMP_ERR_ENV_DYNAMIC,
	GOMP_ERR_ENV_NESTED,
	GOMP_ERR_NUM_THREAD_POSITIVE,

	GOMP_ERR_MAX
};

enum schedule_kind
{
	GOMP_DYNAMIC, 
	GOMP_GUIDED, 
	GOMP_RUNTIME, 
	GOMP_STATIC
};

/* structure containing the current state */
struct t_gomp_state 
{
	enum schedule_kind schedule;
	unsigned int schedule_chunksize;
	unsigned int default_number_of_threads;
	boolean dynamic_adjustment;
	boolean nested_parallism;
};

/* ------------------------------------------------------------------------- */
/* DEFAULT STATE */

#define GOMP_DEFAULT_SCHEDULE GOMP_DYNAMIC
#define GOMP_DEFAULT_SCHEDULE_CHUNKSIZE 1
#define GOMP_DEFAULT_NUMBER_THREADS 1
#define GOMP_DEFAULT_DYNAMIC_ADJUSTMENT TRUE
#define GOMP_DEFAULT_NESTED_PARALLISM FALSE

/* ------------------------------------------------------------------------- */

extern struct t_gomp_state gomp_state; 

extern void gomp_strip( char *s );
extern int gomp_init( void );
extern void gomp_print_state ( FILE *out, char *indent );

extern int gomp_status;
extern int gomp_is_debug( void );
extern void gomp_set_debug( int debug );
extern char *gomp_get_errstr( int err );

#ifdef __cplusplus
}
#endif

#endif /* _GOMP_H */

/* ------------------------------------------------------------------------- */

